from datetime import datetime

from pymongo import MongoClient
from random import randrange
import os
from UtilsGit import UtilsGit
from pathlib import Path


##  Classe utils.
# Permet de stocker les fonctions utilitaires "transversales"
#


class Utils:
    statdb = None
    chemin = "/home/passerat/Stage/Squelette/"

    # chemin = "/home/jeremie/Stage/Squelette/"

    ##   Filtrage de la saisie utilisateur pour le choix d'un auteur
    # @param Le numéro d'auteur choisi
    @staticmethod
    def switchAuteur(argument):
        switcher = {
            1: "Auteur 1",
            2: "Auteur 2",
            3: "Auteur 3",
        }
        return switcher.get(argument, "Auteur inconnu")

    ##
    #    Connexion avec la base de données
    #    recherche -> find and find_one
    #     insertion -> insert_one and insert_many
    #     maj -> update_one, update_many and replace_one
    #     suppression -> delete_one and delete_many
    @staticmethod
    def initialiseMongo():
        client = MongoClient('localhost', 27017)

        ## extraction d'une base de données. Si elle n'existe pas, elle esst créée
        Utils.statdb = client.squelette

    ## Création d'utilisateur
    # @param nomUtilisateur Le nom de l'utilisateur
    @staticmethod
    def ajouterUtilisateur(nomUtilisateur):
        Utils.statdb.utilisateur.insert_one(
            {"nom": nomUtilisateur, "note": randrange(0, 10, 1), "favoris": []})

    ## Vérification de la présence d'un utilisateur en base
    # @param nomUtilisateurLe nom de l'utilisateur
    # @return True si Utilisateur présent, False sinon
    @staticmethod
    def presenceUtilisateur(nomUtilisateur):
        # print("Je suis dans présence utilisateur pour ", nomUtilisateur)
        nmbre = Utils.statdb.utilisateur.find({"nom": nomUtilisateur})
        # print("Et j'en ai trouvé : ", nmbre.count())

        return nmbre.count() == 1

    ##    Creation d'un contenu
    #   @param nomFichier le nom du fichier importé
    #   @param auteur l'auteur qui importe le fichier
    @staticmethod
    def ajouterContenu(nomFichier, auteur):

        max_id = Utils.getMaxIdContenus()
        if max_id is None:
            max_id = 0

        print("nomFichier : ", Utils.chemin + "dossier" + nomFichier + "/")

        cheminFichier = Utils.chemin + "dossier" + nomFichier + "/"

        # Un contenu doit être dans un répertoire isolé pour pouvoir le manipuler en versionnage
        Path(cheminFichier).mkdir(parents=True, exist_ok=True)
        os.system("cp " + Utils.chemin + nomFichier + " " + cheminFichier + nomFichier)

        commit = UtilsGit.commit(cheminFichier, nomFichier, "Version 1")
        print("commit : ", commit)

        Utils.statdb.contenu.insert_one(
            {"id": max_id + 1, "auteur": auteur, "note": randrange(0, 10, 1),
             "fichier": "dossier" + nomFichier + "/" + nomFichier,
             "versionEnCours": {"numero": 1, "date": datetime.now().strftime("%d/%m/%y"),
                                "popularite": 0, "commit": commit},
             "tags_fixes": {"1": "tag1", "2": "tag1", "3": "tag1", "4": "tag1",
                            "5": "tag1"},
             "tags": {"1": "tag1", "2": "tag1", "3": "tag1", "4": "tag1", "5": "tag1"}})
        print("Le contenu a bien été ajouté pour l'utilisateur ", auteur)

    ##  Consultation de tous les contenus publics :
    #    Si panier est égal à True, on sort les contenus de l'auteur
    #    Si panier est égal à False, on sort toutes les questions sauf celles de l'auteur
    #    (Nb: La problématique des contenus "en cours de modification" sera traitée ultérieurement)
    # @return les contenus de l'auteur ou tous les autres

    @staticmethod
    def consulterContenu(auteur, panier=False):
        if panier:
            return Utils.statdb.contenu.find({"auteur": auteur})
        else:
            return Utils.statdb.contenu.find({"auteur": {"$ne": auteur}})

    ##    Consultation de tous les contenus, sans distinction
    #      (utile temporairement pour la création d'évaluations)
    # @return tous les contenus présents

    @staticmethod
    def consulterTousLesContenu():
        return Utils.statdb.contenu.find()

    ## Suppression d'un contenu (dissociation avec son auteur)
    # @param numContenu l'id du contenu à dissocier

    @staticmethod
    def supprimerContenu(numContenu):
        Utils.statdb.contenu.update({"id": numContenu}, {"$set": {"auteur": "Aucun"}})

    ## Permet d'associer un contenu avec le panier d'un auteur tiers
    # @param idContenu l'id associé au contenu
    # @param auteur le nom de l'auteur qui désire un favori

    @staticmethod
    def mettreContenuEnFavori(idContenu, auteur):
        # print("Le contenu est dans la version ", Utils.getVersionContenuavecId(idContenu))
        Utils.statdb.utilisateur.update({"nom": auteur}, {
            "$push": {"favoris": [{"contenu": int(idContenu)}, {"version": Utils.getVersionContenuavecId(idContenu)}]}})

    ## Permet de dissocier un contenu du panier d'un auteur tiers
    # Si cet auteur était le seul à posséder une question "anonyme", elle est supprimée
    # @param idContenu l'identifiant du contenu favori
    @staticmethod
    def supprimerFavori(idContenu, auteur, version):
        Utils.statdb.utilisateur.update({"nom": auteur},
                                        {"$pull": {"favoris": [{"contenu": idContenu}, {"version": version}]}})

    ## Permet de savoir si un contenu possède un auteur légitime
    # @param idContenu l'identifiant du contenu
    @staticmethod
    def isAnonyme(idContenu):
        print("Is anonyme : ", Utils.statdb.contenu.find_one({"id": idContenu})["auteur"])
        return Utils.statdb.contenu.find_one({"id": idContenu})["auteur"] == "Aucun"

    ## Récupérer un contenu à partir de son identifiant et sa version
    # @param idContenu l'id associé au contenu
    # @param version version désirée
    # @return Le contenu lié à l'id

    @staticmethod
    def getContenuavecIdEtVersion(idContenu, version):

        # a décommenter quand git sera actif
        # Utils.changerVersion(idContenu, version, "")
        return Utils.statdb.contenu.find_one({"id": idContenu})
        # return Utils.statdb.contenu.find_one({"$and": [
        #     {"id": idContenu},
        #     {"$or": [{"versionEnCours.numero": version}, {"historique.numero": version}]}
        # ]
        # })

    ## Récupérer un contenu à partir de son identifiant
    # @param idContenu l'id associé au contenu
    # @return Le contenu lié à l'id
    @staticmethod
    def getContenuavecId(idContenu):
        # Id unique donc find_one utilisable
        return Utils.statdb.contenu.find_one({"id": idContenu})

    ## Récupérer la version en cours d'un contenu à partir de son identifiant
    # @param idContenu l'id associé au contenu
    # @return La version en cours du contenu lié à l'id
    @staticmethod
    def getVersionContenuavecId(idContenu):
        # print("Je veux le contenu : ", int(idContenu))
        return int(Utils.getContenuavecId(int(idContenu))["versionEnCours"]["numero"])

    ##  Récupérer l'id maximal (pour les contenus)
    # permet de s'assurer que les id de contenu soient uniques

    @staticmethod
    def getMaxIdContenus():
        ## Fonction permettant de récupérer l'id max présent en base (pour pouvoir incrémenter un nouveau contenu).
        element = Utils.statdb.contenu.find().sort("id", -1).limit(1)

        for elt in element:
            # print("id : ", elt["id"])
            return int(elt["id"])

    ##  Récupérer l'id maximal (pour les evaluations)
    # permet de s'assurer que les id de contenu soient uniques

    @staticmethod
    def getMaxIdEvaluations():
        ## Fonction permettant de récupérer l'id max présent en base (pour pouvoir incrémenter un nouveau contenu).
        element = Utils.statdb.evaluation.find().sort("idEval", -1).limit(1)

        for elt in element:
            # print("id : ", elt["id"])
            return int(elt["idEval"])

    ##  Récupérer l'id maximal (pour les contenus en cours de modification)
    # permet de s'assurer que les id de contenu soient uniques

    @staticmethod
    def getMaxIdContenusEnCours():
        ## Fonction permettant de récupérer l'id max présent en base (pour pouvoir incrémenter un nouveau contenu).
        element = Utils.statdb.encours.find().sort("idContenuEC", -1).limit(1)

        for elt in element:
            print("id : ", elt)
            return int(elt["idContenuEC"])

    ## Récupérer les favoris d'un utilisateur
    # @param auteur le nom de l'utilisateur
    # @return les favoris de l'utilisateur 'auteur'

    @staticmethod
    def getFavoris(auteur):
        return Utils.statdb.utilisateur.find_one({"nom": auteur})["favoris"]

    ##  Créer une évaluation
    #   @param contenus les numeros des contenus associés à l'évaluation
    #   @auteur l'auteur qui créée l'évaluation

    @staticmethod
    def creerEvaluation(contenus, auteur):

        # création de l'évaluation
        max_id_eval = Utils.getMaxIdEvaluations()
        if max_id_eval is None:
            max_id_eval = 0

        Utils.statdb.evaluation.insert_one(
            {"idEval": max_id_eval + 1, "date": datetime.now().strftime("%d/%m/%y"), "auteur": auteur,
             "contenu(s)": contenus})
        # on renseigne l'évaluation dans les contenus associés
        for i in contenus:
            Utils.statdb.contenu.update({"id": i}, {"$push": {"evals": maxIdEval + 1}})

    ## Récupérer les évaluations d'un auteur
    #   @param auteur le nom de l'auteur
    #   @return les évaluations de l'auteur

    @staticmethod
    def getEvaluations(auteur):
        return Utils.statdb.evaluation.find({"auteur": auteur})

    ## Modifier un contenu
    #   @param idContenu contenu à modifier
    #   @param auteur l'utilisateur désirant modifier la question
    #   @param isEnCours Vrai si modification d'un contenu en cours
    @staticmethod
    def modifierContenu(idContenu, auteur, enCours=False):

        # le chemin du fichier restera tjrs le même
        contenu = Utils.statdb.contenu.find_one({"id": int(idContenu)})

        Utils.modification(Utils.chemin + contenu["fichier"], idContenu, auteur, not enCours)

    @staticmethod
    ## affichage d'un contenu de fichier
    def affichage(chemin):
        with open(chemin, "r") as fichier:
            texte = fichier.read()

        texte = texte.strip()
        print("Le fichier (v", int(texte[-1]), ") : " + texte)
        return texte

    ## modification (directe ici) d'un fichier
    # @param chemin chemin d'acces au contenu
    # @param id id du contenu
    # @param auteur auteur qui fait la modification du contenu
    # @param modifPremiere permet de déterminer si on est sur une premiere modification du contenu ou si on remodifie un contenu en cours

    @staticmethod
    def modification(chemin, idContenu, auteur, modifPremiere=True):

        print("Le chemin du fichier est : ", chemin)

        # on récupère la fin du fichier (la version)
        fin = Utils.affichage(chemin)
        version = int(fin[-1])
        if version == 1:
            ecart = 2
        else:
            ecart = 1
        repertoire = chemin[:chemin.rfind("/") + 1]
        nomFichier = chemin[chemin.rfind("/") + 1:]

        print("nomFichier :", nomFichier)
        print("chemin :", chemin)
        # exit(0)

        if modifPremiere:
            # on créée une nouvelle branche et on va dedans
            nomBranche = "version" + str(version + 1)
            print("Je veux la branche", nomBranche)
            UtilsGit.embranchement(repertoire, nomBranche)
            UtilsGit.changerVersionBranche(repertoire, nomBranche)
        # si on est pas en "modifPremiere" on est déja dans la bonne branche.

        # 2. on remplace le dernier caractère du fichier en l'incrémentant de 1
        with open(chemin, "r+") as fichier:
            # pos = fichier.tell()
            # pos = pos - 1
            fichier.seek(0, 2)
            # seek to end of file; f.seek(0, os.SEEK_END) is legal

            fichier.seek(fichier.tell() - ecart, 0)
            fichier.truncate()
            fichier.write(str(version + 1))
            # fichier.seek(0, 2)
            # print("pront:", fichier.read())
            fichier.close()
        Utils.affichage(chemin)
        UtilsGit.commit(repertoire, nomFichier, "modification")

        # si on est dans le cas d'une modification 1ere d'un contenu, on l'entre en bd (sinon il y est déja)
        if modifPremiere:
            max_id_encours = Utils.getMaxIdContenusEnCours()
            if max_id_encours is None:
                max_id_encours = 0
            Utils.statdb.encours.insert_one(
                {"idContenuEC": max_id_encours + 1, "date": datetime.now().strftime("%d/%m/%y"), "auteur": auteur,
                 "reference": int(idContenu), "version": version, "branche": nomBranche})

    ## Mettre à jour un contenu (branche principale)
    @staticmethod
    def majContenu(chemin, idContenu, nomBranche):

        repertoire = chemin[:chemin.rfind("/") + 1]
        fin = Utils.affichage(chemin)
        version = int(fin[-1])

        # Mise à jour du répertoire git
        idCommit = UtilsGit.mergeVersion(repertoire, nomBranche)

        # Mise à jour des métadonnées - ok
        version_en_cours = Utils.statdb.contenu.find_one({"id": idContenu})["versionEnCours"]
        Utils.statdb.contenu.update_one({"id": int(idContenu)}, {"$push": {"historique": version_en_cours}})
        Utils.statdb.contenu.update_one({"id": int(idContenu)}, {
            "$set": {"versionEnCours": {"numero": version, "date": datetime.now().strftime("%d/%m/%y"),
                                        "popularite": 0, "commit": idCommit}}})

    ## Supprimer une évaluation (définitif)
    # @param idEval l'id de l'évaluation à supprimer
    @staticmethod
    def supprimerEvaluation(idEval):
        Utils.statdb.evaluation.delete_one({"idEval": idEval})

    ## Fonction pour savoir si un contenu est favori chez qqun
    # @param idContenu
    @staticmethod
    def existeEnFavori(idContenu):
        return Utils.statdb.utilisateur.find(
            {"favoris": {"$elemMatch": {"$elemMatch": {"contenu": idContenu}}}}).count() != 0

    ## On entre l'id d'un contenu favori pour un auteur, on veut récupérer en sortie la version associée
    # @param idContenu
    @staticmethod
    def getVersionFavoriById(idContenu, auteur):
        favoris = (Utils.statdb.utilisateur.find_one({"nom": auteur})["favoris"])
        print(type(favoris))
        for i in range(len(favoris)):
            fav = favoris[i]
            if int(fav[0]["contenu"]) == idContenu:
                return int(fav[1]["version"])
            else:
                return 0

    ## Suppression définitive d'un contenu
    @staticmethod
    def aneantirContenu(idContenu):
        Utils.statdb.contenu.delete_one({"id": idContenu})

    ## permet de récupérer tous les contenus en cours pour un auteur
    # @param auteur le nom de l'auteur

    @staticmethod
    def getContenusEnCours(auteur):
        return Utils.statdb.encours.find({"auteur": auteur})

    ## permet de récupérer le contenu originel associé à un contenu en cours
    # @param idContenuEnCours l'id du contenu en cours de modification

    @staticmethod
    def getIdContenuAssocieAUnEnCours(idContenuEnCours):
        print("L'id en cours que j'ai est", idContenuEnCours, " - ", type(idContenuEnCours))
        return Utils.statdb.encours.find_one({"idContenuEC": idContenuEnCours})["reference"]

    ## récupérer le chemin d'un fichier associé à un contenu
    # @param idContenu l'id du contenu

    @staticmethod
    def getCheminDuContenu(idContenu):
        return Utils.chemin + Utils.statdb.contenu.find_one({"id": idContenu})["fichier"]

    ## permet de changer de version pour un contenu (modification de la branche / du commit en fonction de la situation)
    # @param idContenu le contenu concerné
    # @param version la version du contenu désirée
    # @param nomBranche changement direct de branche -> utile pour naviguer parmi les versions "en cours"
    @staticmethod
    def changerVersion(idContenu, version, nomBranche):
        # on recupere le chemin du fichier associé au contenu (dans les 2 cas)
        chemin = Utils.getCheminDuContenu(idContenu)
        repertoire = chemin[:chemin.rfind("/") + 1]
        if version != 0:
            # on change le repertoire récupéré ci dessus dans la bonne version (si on travaille avec une version)
            idCommit = Utils.getCommitPourUneVersion(idContenu, version)
            UtilsGit.changerVersionCommit(repertoire, idCommit)
        else:
            UtilsGit.changerVersionBranche(repertoire, nomBranche)

    ## permet de recuperer un commit particulier lié avec une version donnee
    # @param idContenu l'id du contenu manipulé
    # @param version le numéro de version désiré
    @staticmethod
    def getCommitPourUneVersion(idContenu, version):
        return 42

    ## supprimer un contenu en cours
    # @param idContenuEnCours id du contenu a supprimer
    @staticmethod
    def supprimerContenuEnCours(idContenuEnCours):
        Utils.statdb.encours.delete_one({"idContenuEC": idContenuEnCours})

    ## récupérer l'auteur initial d'un contenu en cours (peut être l'auteur effectuant la modification ou non)
    # @param idContenuEnCours l'id du contenu en cours de modification
    @staticmethod
    def getAuteurContenuEnCours(idContenuEnCours):
        id_contenu = Utils.getIdContenuAssocieAUnEnCours(idContenuEnCours)
        return Utils.statdb.contenu.find_one({"id": id_contenu})["auteur"]

    ## récupérer la branche d'un contenu en cours
    # @param idContenuEnCours l'id du contenu en cours de modification

    @staticmethod
    def getBrancheContenuEnCours(idContenuEnCours):
        # id_contenu = Utils.getIdContenuAssocieAUnEnCours(idContenuEnCours)
        return Utils.statdb.encours.find_one({"idContenuEC": idContenuEnCours})["branche"]

    ## A NE JAMAIS UTILISER SANS REFLECHIR
    # Suppression integrale de la base de données & recreation des 3 auteurs
    @staticmethod
    def toutPeter():
        Utils.statdb.contenu.delete_many({})
        Utils.statdb.utilisateur.delete_many({})
        Utils.statdb.evaluation.delete_many({})
        Utils.statdb.encours.delete_many({})
        os.system("rm -rf dossierContenu*/")
        Utils.ajouterUtilisateur("Auteur 1")
        Utils.ajouterUtilisateur("Auteur 2")
        Utils.ajouterUtilisateur("Auteur 3")

    ## creer un contenu à partir d'un contenu en cours
    @staticmethod
    def creerNouveauContenu(chemin, auteurEnCours, idContenuEnCours):
        print("chemin : ", chemin)

        # hard codé pour l'exemple, sera adaptatif dans le futur
        max_id = 3

        dossierFichier = "dossierContenu" + str(max_id) + "/"
        cheminFichier = Utils.chemin + dossierFichier
        nomFichier = chemin[chemin.rfind("/") + 1:]

        print("Chemin fichier : ", cheminFichier)
        print("Dossier Fichier : ", dossierFichier)
        print("NomFichier :", nomFichier)
        Path(cheminFichier).mkdir(parents=True, exist_ok=True)
        os.system("cp " + chemin + " " + cheminFichier)
        os.system("mv " + cheminFichier + nomFichier + " " + cheminFichier + "Contenu" + str(max_id))
        # exit(0)
        commit = UtilsGit.commit(cheminFichier, "Contenu" + str(max_id), "Version 1")

        Utils.statdb.contenu.insert_one(
            {"id": max_id + 1, "auteur": auteurEnCours, "note": randrange(0, 10, 1),
             "fichier": "dossierContenu" + str(max_id) + "/Contenu" + str(max_id),
             "versionEnCours": {"numero": 1, "date": datetime.now().strftime("%d/%m/%y"),
                                "popularite": 0, "commit": commit},
             "tags_fixes": {"1": "tag1", "2": "tag1", "3": "tag1", "4": "tag1",
                            "5": "tag1"},
             "tags": {"1": "tag1", "2": "tag1", "3": "tag1", "4": "tag1", "5": "tag1"}})
        print("Le contenu a bien été ajouté pour l'utilisateur ", auteurEnCours)

        Utils.supprimerContenuEnCours(idContenuEnCours)

    ## fonction qui permet de savoir si un contenu fait partie des favoris d'un auteur
    # On fraude un peu en utilisant  getVersionFavoriById -> si la sortie est à 0 ben le contenu n'est pas un favori
    @staticmethod
    def isFavoriDeAuteur(auteur, idContenu):
        return Utils.getVersionFavoriById(idContenu, auteur) != 0
